@@ -3,8 +3,7 @@ require_relative '../pcmk_common'
 Puppet::Type.type(:pcmk_resource).provide(:default) do
   desc 'A base resource definition for a pacemaker resource'
 
-  ### overloaded methods
-  def create
+  def build_pcs_resource_cmd(update=false)
     resource_params = @resource[:resource_params]
     meta_params = @resource[:meta_params]
     op_params = @resource[:op_params]
@@ -14,13 +13,6 @@ Puppet::Type.type(:pcmk_resource).provide(:default) do
     location_rule = @resource[:location_rule]
     bundle = @resource[:bundle]
 
-    # We need to probe the existance of both location and resource
-    # because we do not know why we're being created (if for both or
-    # only for one)
-    did_location_exist = location_exists?
-    did_resource_exist = resource_exists?
-    Puppet.debug("Create: resource exists #{did_resource_exist} location exists #{did_location_exist}")
-
     suffixes = 0
     if clone_params then suffixes +=1 end
     if master_params then suffixes +=1 end
@@ -29,11 +21,19 @@ Puppet::Type.type(:pcmk_resource).provide(:default) do
       raise(Puppet::Error, "May only define one of clone_params, "+
             "master_params and group_params")
     end
+    if update
+      create_cmd = ' update '
+    else
+      create_cmd = ' create '
+    end
 
     # Build the 'pcs resource create' command.  Check out the pcs man page :-)
-    cmd = 'resource create ' + @resource[:name]+' ' +@resource[:resource_type]
+    cmd = 'resource' + create_cmd + @resource[:name] + ' ' + @resource[:resource_type]
     if @resource[:resource_type] == 'remote'
-      @resource[:remote_address] ? cmd += ' server=' + @resource[:remote_address] :
+      if not_empty_string(@resource[:remote_address])
+        cmd += ' server=' + @resource[:remote_address]
+      end
+      # reconnect_interval always has a default
       cmd += " reconnect_interval=#{@resource[:reconnect_interval]}"
     end
     if not_empty_string(resource_params)
@@ -65,31 +65,62 @@ Puppet::Type.type(:pcmk_resource).provide(:default) do
         end
       end
     end
+    cmd
+  end
 
-    # If both the resource and the location do not exist, we create them both
-    # if a location_rule is specified otherwise only the resource
-    if not did_location_exist and not did_resource_exist
-      if location_rule
+  ### overloaded methods
+  def initialize(*args)
+    super(*args)
+    Puppet.debug("puppet-pacemaker: initialize()")
+    # Hash to store the existance state of each resource or location
+    @resources_state = {}
+    @locations_state = {}
+  end
+
+  def create_resource_and_location(location_rule, needs_update=false)
+    if needs_update then
+      cmd = build_pcs_resource_cmd(update=true)
+      pcmk_update_resource(@resource, cmd, '', @resource[:update_settle_secs])
+    else
+      cmd = build_pcs_resource_cmd()
+      if location_rule then
         pcs('create', @resource[:name], "#{cmd} --disabled", @resource[:tries],
             @resource[:try_sleep], @resource[:verify_on_create], @resource[:post_success_sleep])
-        location_rule_create(location_rule)
+        location_rule_create()
         pcs('create', @resource[:name], "resource enable #{@resource[:name]}", @resource[:tries],
             @resource[:try_sleep], @resource[:verify_on_create], @resource[:post_success_sleep])
       else
         pcs('create', @resource[:name], cmd, @resource[:tries],
             @resource[:try_sleep], @resource[:verify_on_create], @resource[:post_success_sleep])
       end
+    end
+  end
+
+  def create
+    # We need to probe the existance of both location and resource
+    # because we do not know why we're being created (if for both or
+    # only for one)
+    did_resource_exist = @resources_state[@resource[:name]] == PCMK_NOCHANGENEEDED
+    did_location_exist = @locations_state[@resource[:name]] == PCMK_NOCHANGENEEDED
+    Puppet.debug("Create: resource exists #{@resources_state[@resource[:name]]} location exists #{@locations_state[@resource[:name]]}")
+    needs_update = @resources_state[@resource[:name]] == PCMK_CHANGENEEDED
+
+    cmd = build_pcs_resource_cmd()
+
+    # If both the resource and the location do not exist, we create them both
+    # if a location_rule is specified otherwise only the resource
+    if not did_location_exist and not did_resource_exist
+      create_resource_and_location(location_rule, needs_update)
     # If the location_rule already existed, we only create the resource
     elsif did_location_exist and not did_resource_exist
-      pcs('create', @resource[:name], cmd, @resource[:tries],
-          @resource[:try_sleep], @resource[:verify_on_create], @resource[:post_success_sleep])
+      create_resource_and_location(false, needs_update)
     # The location_rule does not exist and the resource does exist
     elsif not did_location_exist and did_resource_exist
       if location_rule
-        location_rule_create(location_rule)
+        location_rule_create()
       end
     else
-      raise Puppet::Error, "Invalid create: #{name} resource exists #{did_resource_exist} "
+      raise Puppet::Error, "Invalid create: #{@resource[:name]} resource exists #{did_resource_exist} "
                            "location exists #{did_location_exist} - location_rule #{location_rule}"
     end
   end
@@ -103,9 +134,14 @@ Puppet::Type.type(:pcmk_resource).provide(:default) do
   end
 
   def exists?
-    did_location_exist = location_exists?
-    did_resource_exist = resource_exists?
-    Puppet.debug("Exists: resource exists #{did_resource_exist} location exists #{did_location_exist}")
+    @locations_state[@resource[:name]] = location_exists?
+    @resources_state[@resource[:name]] = resource_exists?
+    did_resource_exist = @resources_state[@resource[:name]] == PCMK_NOCHANGENEEDED
+    did_location_exist = @locations_state[@resource[:name]] == PCMK_NOCHANGENEEDED
+    Puppet.debug("Exists: resource #{@resource[:name]} exists "\
+                 "#{@resources_state[@resource[:name]]} "\
+                 "location exists #{@locations_state[@resource[:name]]} "\
+                 "resource deep_compare: #{@resource[:deep_compare]}")
     if did_resource_exist and did_location_exist
       return true
     end
@@ -116,7 +152,13 @@ Puppet::Type.type(:pcmk_resource).provide(:default) do
     cmd = 'resource show ' + @resource[:name] + ' > /dev/null 2>&1'
     ret = pcs('show', @resource[:name], cmd, @resource[:tries],
               @resource[:try_sleep], @resource[:verify_on_create], @resource[:post_success_sleep])
-    return ret == false ? false : true
+    if ret == false then
+      return PCMK_NOTEXISTS
+    end
+    if @resource[:deep_compare] and pcmk_resource_has_changed?(@resource, build_pcs_resource_cmd(update=true), '') then
+      return PCMK_CHANGENEEDED
+    end
+    return PCMK_NOCHANGENEEDED
   end
 
   def location_exists?
@@ -124,7 +166,7 @@ Puppet::Type.type(:pcmk_resource).provide(:default) do
     # If no location_rule is specified then we treat it as if it
     # always exists
     if not @resource[:location_rule]
-      return true
+      return PCMK_NOCHANGENEEDED
     end
     if bundle
       constraint_name = 'location-' + bundle
@@ -139,36 +181,11 @@ Puppet::Type.type(:pcmk_resource).provide(:default) do
     cmd = "constraint list | grep #{constraint_name} > /dev/null 2>&1"
     ret = pcs('show', @resource[:name], cmd, @resource[:tries],
               @resource[:try_sleep], @resource[:verify_on_create], @resource[:post_success_sleep])
-    return ret == false ? false : true
+    return ret == false ? PCMK_NOTEXISTS : PCMK_NOCHANGENEEDED
   end
 
-  def location_rule_create(location_rule)
-    # The name that pcs will create is location-<name>[-{clone,master}]
-    bundle = @resource[:bundle]
-    location_cmd = 'constraint location '
-    if bundle
-      location_cmd += bundle
-    else
-      location_cmd += @resource[:name]
-      if clone_params
-        location_cmd += '-clone'
-      elsif master_params
-        location_cmd += '-master'
-      end
-    end
-    location_cmd += ' rule'
-    if location_rule['resource_discovery']
-      location_cmd += " resource-discovery=#{location_rule['resource_discovery']}"
-    end
-    if location_rule['score']
-      location_cmd += " score=#{location_rule['score']}"
-    end
-    if location_rule['score_attribute']
-      location_cmd += " score-attribure=#{location_rule['score_attribute']}"
-    end
-    if location_rule['expression']
-      location_cmd += " " + location_rule['expression'].join(' ')
-    end
+  def location_rule_create()
+    location_cmd = build_pcs_location_rule_cmd(@resource)
     Puppet.debug("location_rule_create: #{location_cmd}")
     pcs('create', @resource[:name], location_cmd, @resource[:tries],
         @resource[:try_sleep], @resource[:verify_on_create], @resource[:post_success_sleep])
